import Snap from 'jest-snapshot';
import { afterAll, beforeEach } from '../../../hooks';
import { SnapshotManager } from './manager';
const { addSerializer } = Snap;
let _manager;
export async function SnapshotPlugin(options) {
    const { rootDir } = options;
    _manager = new SnapshotManager({
        rootDir,
        update: options.update,
    });
    _manager.snapshotResolver = await Snap.buildSnapshotResolver({
        transform: [],
        rootDir,
    });
    return function (chai, utils) {
        beforeEach((task) => {
            var _a;
            _manager.setContext({
                file: ((_a = task.file) === null || _a === void 0 ? void 0 : _a.filepath) || task.name,
                title: task.name,
                fullTitle: [task.suite.name, task.name].filter(Boolean).join(' > '),
            });
        });
        afterAll(() => {
            _manager.saveSnap();
            _manager.report();
        });
        for (const key of ['matchSnapshot', 'toMatchSnapshot']) {
            utils.addMethod(chai.Assertion.prototype, key, function (message) {
                const expected = utils.flag(this, 'object');
                _manager.assert(expected, message);
            });
        }
        chai.expect.addSnapshotSerializer = addSerializer;
    };
}
