import { context } from './context';
export const defaultSuite = suite('');
function getCurrentSuite() {
    return context.currentSuite || defaultSuite;
}
export const test = (name, fn) => getCurrentSuite().test(name, fn);
test.skip = (name, fn) => getCurrentSuite().test.skip(name, fn);
test.only = (name, fn) => getCurrentSuite().test.only(name, fn);
test.todo = (name) => getCurrentSuite().test.todo(name);
function createSuiteCollector(mode, suiteName, factory) {
    const queue = [];
    const factoryQueue = [];
    const collector = {
        name: suiteName,
        mode,
        test,
        collect,
        clear,
    };
    function collectTask(name, fn, mode) {
        queue.push({
            name,
            mode,
            suite: {},
            state: mode !== 'run' ? mode : undefined,
            fn,
        });
    }
    function test(name, fn) {
        collectTask(name, fn, mode);
    }
    test.skip = (name, fn) => collectTask(name, fn, 'skip');
    test.only = (name, fn) => collectTask(name, fn, 'only');
    test.todo = (name) => collectTask(name, () => { }, 'todo');
    function clear() {
        queue.length = 0;
        factoryQueue.length = 0;
    }
    async function collect(file) {
        factoryQueue.length = 0;
        if (factory)
            await factory(test);
        const tasks = [...factoryQueue, ...queue];
        const suite = {
            name: collector.name,
            mode: collector.mode,
            tasks,
            file,
        };
        tasks.forEach((task) => {
            task.suite = suite;
            if (file)
                task.file = file;
        });
        return suite;
    }
    context.currentSuite = collector;
    context.suites.push(collector);
    return collector;
}
export function suite(suiteName, factory) {
    return createSuiteCollector('run', suiteName, factory);
}
suite.skip = (suiteName, factory) => createSuiteCollector('skip', suiteName, factory);
suite.only = (suiteName, factory) => createSuiteCollector('only', suiteName, factory);
suite.todo = (suiteName) => createSuiteCollector('todo', suiteName);
// alias
export const describe = suite;
export const it = test;
// utils
export function clearContext() {
    context.suites.length = 0;
    defaultSuite.clear();
    context.currentSuite = defaultSuite;
}
